# -*- coding: utf-8 -*-
from themoviedb import schemas, utils
from themoviedb.routes.base import Base


class TV(Base):
    def __init__(self, tv_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.tv_id = tv_id

    async def details(self, *, append_to_response: str = None) -> schemas.TV:
        """Get the primary TV show details by id.

        See more: https://developers.themoviedb.org/3/tv/get-tv-details
        """
        data = await self.request(f"tv/{self.tv_id}", append_to_response=append_to_response)
        return utils.as_dataclass(schemas.TV, data)

    async def aggregate_credits(self) -> schemas.Credits:
        """Get the credits (cast and crew) that have been added to a TV show.

        This call differs from the main credits call in that it does
        not return the newest season but rather, is a view of all the
        entire cast & crew for all episodes belonging to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-aggregate-credits
        """
        data = await self.request(f"tv/{self.tv_id}/aggregate_credits")
        return utils.as_dataclass(schemas.Credits, data)

    async def alternative_titles(self, *, country: str = None) -> schemas.AlternativeTitles:
        """Get all alternative titles for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-alternative-titles
        """
        data = await self.request(f"tv/{self.tv_id}/alternative_titles", country=country)
        return utils.as_dataclass(schemas.AlternativeTitles, data)

    async def credits(self) -> schemas.Credits:
        """Get the credits (cast and crew) that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-credits
        """
        data = await self.request(f"tv/{self.tv_id}/credits")
        return utils.as_dataclass(schemas.Credits, data)

    async def external_ids(self) -> schemas.ExternalIDs:
        """Get the external ids for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-external-ids
        """
        data = await self.request(f"tv/{self.tv_id}/external_ids")
        return utils.as_dataclass(schemas.ExternalIDs, data)

    async def images(self) -> schemas.Images:
        """Get the images that belong to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-images
        """
        data = await self.request(f"tv/{self.tv_id}/images")
        return utils.as_dataclass(schemas.Images, data)

    async def keywords(self) -> schemas.Keywords:
        """Get the keywords that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-keywords
        """
        data = await self.request(f"tv/{self.tv_id}/keywords")
        return utils.as_dataclass(schemas.Keywords, data)

    async def recommendations(self, *, page: int = 1) -> schemas.TVs:
        """Get the list of TV show recommendations for this item.

        See more: https://developers.themoviedb.org/3/tv/get-tv-recommendations
        """
        data = await self.request(f"tv/{self.tv_id}/recommendations", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def reviews(self, *, page: int = 1) -> schemas.Reviews:
        """Get the reviews for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-reviews
        """
        data = await self.request(f"tv/{self.tv_id}/reviews", page=page)
        return utils.as_dataclass(schemas.Reviews, data)

    async def similar(self, *, page: int = 1) -> schemas.TVs:
        """Get a list of similar TV shows. These items are assembled by looking at keywords and genres.

        See more: https://developers.themoviedb.org/3/tv/get-similar-tv-shows
        """
        data = await self.request(f"tv/{self.tv_id}/similar", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def translations(self) -> schemas.Translations:
        """Get a list of the translations that exist for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-translations
        """
        data = await self.request(f"tv/{self.tv_id}/translations")
        return utils.as_dataclass(schemas.Translations, data)

    async def videos(self, *, page: int = 1) -> schemas.Videos:
        """Get the videos that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-videos
        """
        data = await self.request(f"tv/{self.tv_id}/videos", page=page)
        return utils.as_dataclass(schemas.Videos, data)

    async def watch_providers(self) -> schemas.WatchProviders:
        """Get a list of the availabilities per country by provider.

        See more: https://developers.themoviedb.org/3/tv/get-tv-watch-providers
        """
        data = await self.request(f"tv/{self.tv_id}/watch/providers")
        return utils.as_dataclass(schemas.WatchProviders, data)


class TVs(Base):

    async def latest(self) -> schemas.TV:
        """Get the most newly created TV show.

        See more: https://developers.themoviedb.org/3/tv/get-latest-tv
        """
        data = await self.request("tv/latest")
        return utils.as_dataclass(schemas.TV, data)

    async def airing_today(self, *, page: int = 1) -> schemas.TVs:
        """Get a list of TV shows that are airing today.

        See more: https://developers.themoviedb.org/3/tv/get-tv-airing-today
        """
        data = await self.request("tv/airing_today", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def on_the_air(self, *, page: int = 1) -> schemas.TVs:
        """Get a list of shows that are currently on the air.

        See more: https://developers.themoviedb.org/3/tv/get-tv-on-the-air
        """
        data = await self.request("tv/on_the_air", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def popular(self, *, page: int = 1) -> schemas.TVs:
        """Get a list of the current popular TV shows on TMDB.

        See more: https://developers.themoviedb.org/3/tv/get-popular-tv-shows
        """
        data = await self.request("tv/popular", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def top_rated(self, *, page: int = 1) -> schemas.TVs:
        """Get a list of the top rated TV shows on TMDB.

        See more: https://developers.themoviedb.org/3/tv/get-top-rated-tv
        """
        data = await self.request("tv/top_rated", page=page)
        return utils.as_dataclass(schemas.TVs, data)



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._result import ResultWithID


@dataclass
class AlternativeName:
    name: Optional[str] = None
    type: Optional[str] = None

    def __str__(self) -> str:
        return self.name


@dataclass
class AlternativeNames(ResultWithID):
    results: Optional[List[AlternativeName]] = None



from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas.credits import Cast, Crew


@dataclass
class Episode:
    id: Optional[int] = None
    air_date: Optional[date] = None
    episode_number: Optional[int] = None
    crew: Optional[List[Crew]] = None
    guest_stars: Optional[List[Cast]] = None
    name: Optional[str] = None
    overview: Optional[str] = None
    production_code: Optional[str] = None
    season_number: Optional[int] = None
    still_path: Optional[str] = None
    vote_average: Optional[float] = None
    vote_count: Optional[int] = None

    def __str__(self) -> str:
        return self.name

    def still_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.still_path}" if self.still_path else None



from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas.episodes import Episode


@dataclass
class Season:
    id: Optional[int] = None
    _id: Optional[str] = None
    air_date: Optional[date] = None
    episodes: Optional[List[Episode]] = None
    name: Optional[str] = None
    overview: Optional[str] = None
    season_id: Optional[int] = None
    poster_path: Optional[str] = None
    season_number: Optional[int] = None

    def __str__(self) -> str:
        return self.name

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialCollection, PartialMovie
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Collection(PartialCollection):
    overview: Optional[str] = None
    parts: Optional[List[PartialMovie]] = None


@dataclass
class Collections(ResultWithPage):
    results: Optional[List[Collection]] = None



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._result import ResultWithID


@dataclass
class Data:
    name: Optional[str] = None
    title: Optional[str] = None
    overview: Optional[str] = None
    homepage: Optional[str] = None
    biography: Optional[str] = None

@dataclass
class Translation:
    iso_3166_1: Optional[str] = None
    iso_639_1: Optional[str] = None
    english_name: Optional[str] = None
    name: Optional[str] = None
    data: Optional[Data] = None

    def __str__(self) -> str:
        return self.name


@dataclass
class Translations(ResultWithID):
    translations: Optional[List[Translation]] = None

    def __post_init__(self) -> None:
        self.results = self.translations



from themoviedb.routes.base import Base
from themoviedb.routes.credit import Credit
from themoviedb.routes.companies import Company
from themoviedb.routes.collections import Collection
from themoviedb.routes.discover import Discover
from themoviedb.routes.episodes import Episode
from themoviedb.routes.find import Find
from themoviedb.routes.genres import Genres
from themoviedb.routes.keywords import Keyword
from themoviedb.routes.movies import Movie, Movies
from themoviedb.routes.people import People, Person
from themoviedb.routes.network import Network
from themoviedb.routes.reviews import Review
from themoviedb.routes.search import Search
from themoviedb.routes.seasons import Season
from themoviedb.routes.trending import Trending
from themoviedb.routes.tv import TV, TVs
from themoviedb.routes.watch_providers import WatchProviders



from enum import Enum


class CreditType(str, Enum):
    cast = "cast"
    crew = "crew"


class MediaType(str, Enum):
    movie = "movie"
    person = "person"
    tv = "tv"


class SizeType(str, Enum):
    w45 = "w45"
    w92 = "w92"
    w154 = "w154"
    w185 = "w185"
    w300 = "w300"
    w342 = "w342"
    w500 = "w500"
    h632 = "h632"
    w780 = "w780"
    w1280 = "w1280"
    original = "original"



# -*- coding: utf-8 -*-
import os

from aiohttp import ClientSession


class Base:
    _TMDB_URL = "https://api.themoviedb.org"
    _TMDB_VERSION = "3"

    def __init__(
        self,
        *,
        session: ClientSession = None,
        language: str = None,
        region: str = None,
    ):
        self._session = session
        self._language = language if language is not None else os.environ.get("TMDB_LANGUAGE", "en-US")
        self._region = region if region is not None else os.environ.get("TMDB_REGION", "US")

    @property
    def session(self) -> ClientSession:
        return self._session

    @session.setter
    def session(self, session: ClientSession) -> None:
        self._session = session

    @property
    def key(self) -> str:
        return os.environ.get("TMDB_KEY")

    @key.setter
    def key(self, key: str) -> None:
        os.environ["TMDB_KEY"] = key

    @property
    def language(self) -> str:
        return self._language

    @language.setter
    def language(self, language: str) -> None:
        self._language = language

    @property
    def region(self) -> str:
        return self._region

    @region.setter
    def region(self, region: str) -> None:
        self._region = region

    @property
    def _host(self) -> str:
        return self._TMDB_URL

    @property
    def _version(self) -> str:
        return self._TMDB_VERSION

    @property
    def _params(self) -> dict:
        return {
            "api_key": self.key,
            "language": self.language,
            "region": self.region,
            "watch_region": self.region,
        }

    async def request(self, path: str, method: str = "GET", **kwargs) -> dict:
        url = f"{self._host}/{self._version}/{path}"
        params = {
            k.replace("__", "."): "true" if v is True else "false" if v is False else v
            for k, v in kwargs.items()
            if v is not None
        }
        params = {**self._params, **params}

        if self._session:
            response = await self.session.request(method, url, params=params)
            data = await response.json()
        else:
            async with ClientSession(raise_for_status=True) as session:
                async with session.request(method, url, params=params) as response:
                    data = await response.json()

        if "watch/providers" in data:
            data["watch_providers"] = data.pop("watch/providers")
        return data



from dataclasses import dataclass
from typing import Dict, List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas._result import Result, ResultWithID


@dataclass
class WatchProviderData:
    display_priority: Optional[int] = None
    logo_path: Optional[str] = None
    provider_name: Optional[str] = None
    provider_id: Optional[int] = None

    def __str__(self) -> str:
        return self.provider_name

    def logo_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.logo_path}" if self.logo_path else None


@dataclass
class WatchProvidersData(Result):
    results: Optional[List[WatchProviderData]] = None


@dataclass
class WatchProvider:
    link: Optional[str] = None
    ads: Optional[List[WatchProviderData]] = None
    buy: Optional[List[WatchProviderData]] = None
    flatrate: Optional[List[WatchProviderData]] = None
    free: Optional[List[WatchProviderData]] = None
    rent: Optional[List[WatchProviderData]] = None

    def __bool__(self) -> bool:
        return bool(self.buy or self.flatrate or self.free)


@dataclass
class WatchProviders(ResultWithID):
    results: Optional[Dict[str, WatchProvider]] = None

    def __bool__(self) -> bool:
        return bool(self.results)

    def __getitem__(self, region: str) -> WatchProvider:
        return self.results[region]

    @property
    def regions(self) -> List[str]:
        return list(self.results.keys())



from themoviedb.schemas._enums import CreditType, MediaType, SizeType
from themoviedb.schemas._partial import (
    KnownFor,
    PartialCollection,
    PartialCompany,
    PartialKeyword,
    PartialMovie,
    PartialPerson,
    PartialTV,
)
from themoviedb.schemas._result import (
    Dates,
    Result,
    ResultWithID,
    ResultWithPage,
)
from themoviedb.schemas.alternative_names import AlternativeName, AlternativeNames
from themoviedb.schemas.alternative_titles import AlternativeTitle, AlternativeTitles
from themoviedb.schemas.changes import Change
from themoviedb.schemas.collections import Collection, Collections
from themoviedb.schemas.companies import Companies, Company
from themoviedb.schemas.credit import Credit
from themoviedb.schemas.credits import (
    Cast,
    CastCombined,
    CastMovie,
    CastTV,
    Credits,
    CreditsCombined,
    CreditsMovie,
    CreditsTV,
    Crew,
    CrewCombined,
    CrewMovie,
    CrewTV,
)
from themoviedb.schemas.external_ids import ExternalIDs
from themoviedb.schemas.episodes import Episode
from themoviedb.schemas.genres import Genre, Genres
from themoviedb.schemas.images import Image, Images
from themoviedb.schemas.keywords import Keyword, Keywords
from themoviedb.schemas.list import ItemList, ItemsList
from themoviedb.schemas.movies import Movie, Movies
from themoviedb.schemas.multi import Multi, Multis, MultiResults
from themoviedb.schemas.networks import Network
from themoviedb.schemas.people import People, Person
from themoviedb.schemas.regions import Region, Regions
from themoviedb.schemas.release_date import ReleaseDate, ReleaseDates
from themoviedb.schemas.review import Review, Reviews
from themoviedb.schemas.seasons import Season
from themoviedb.schemas.translations import Translation, Translations
from themoviedb.schemas.tv import TV, TVs
from themoviedb.schemas.videos import Video, Videos
from themoviedb.schemas.watch_providers import (
    WatchProvider,
    WatchProviderData,
    WatchProviders,
    WatchProvidersData,
)



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas._result import ResultWithPage


@dataclass
class ItemList:
    id: int
    description: Optional[str] = None
    favorite_count: Optional[int] = None
    item_count: Optional[int] = None
    iso_639_1: Optional[str] = None
    list_type: Optional[str] = None
    name: Optional[str] = None
    poster_path: Optional[str] = None

    def __str__(self) -> str:
        return self.name

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class ItemsList(ResultWithPage):
    results: Optional[List[ItemList]] = None



from dataclasses import dataclass
from typing import Optional

from themoviedb.schemas._enums import SizeType


@dataclass
class Network:
    id: int
    headquarters: Optional[str] = None
    homepage: Optional[str] = None
    logo_path: Optional[str] = None
    name: Optional[str] = None
    origin_country: Optional[str] = None

    def __str__(self) -> str:
        return self.name

    def logo_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.logo_path}" if self.logo_path else None



from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._enums import MediaType, SizeType


@dataclass
class PartialCompany:
    id: int
    logo_path: Optional[str] = None
    name: Optional[str] = None

    def __str__(self) -> str:
        return self.name

    def logo_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.logo_path}" if self.logo_path else None


@dataclass
class PartialCollection:
    id: int
    backdrop_path: Optional[str] = None
    name: Optional[str] = None
    poster_path: Optional[str] = None

    def __str__(self) -> str:
        return self.name

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class PartialKeyword:
    id: int
    name: Optional[str] = None

    def __str__(self) -> str:
        return self.name


@dataclass
class PartialMovie:
    id: int
    poster_path: Optional[str] = None
    adult: Optional[bool] = None
    overview: Optional[str] = None
    release_date: Optional[date] = None
    genre_ids: Optional[List[int]] = None
    original_title: Optional[str] = None
    original_language: Optional[str] = None
    title: Optional[str] = None
    backdrop_path: Optional[str] = None
    popularity: Optional[float] = None
    vote_count: Optional[int] = None
    video: Optional[bool] = None
    vote_average: Optional[float] = None
    media_type: Optional[MediaType] = MediaType.movie

    def __str__(self) -> str:
        return self.title or self.original_title

    @property
    def year(self) -> Optional[int]:
        return self.release_date.year if self.release_date else None

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class PartialTV:
    id: int
    poster_path: Optional[str] = None
    adult: Optional[bool] = None
    popularity: Optional[float] = None
    backdrop_path: Optional[str] = None
    vote_average: Optional[float] = None
    overview: Optional[str] = None
    first_air_date: Optional[date] = None
    origin_country: Optional[List[str]] = None
    genre_ids: Optional[List[int]] = None
    original_language: Optional[str] = None
    vote_count: Optional[int] = None
    name: Optional[str] = None
    original_name: Optional[str] = None
    media_type: Optional[MediaType] = MediaType.tv

    def __str__(self) -> str:
        return self.name or self.original_name

    @property
    def year(self) -> Optional[int]:
        return self.first_air_date.year if self.first_air_date else None

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class PartialSeason:
    id: int = None
    air_date: Optional[date] = None
    episode_count: Optional[int] = None
    name: Optional[str] = None
    overview: Optional[str] = None
    poster_path: Optional[str] = None
    season_number: Optional[int] = None

    def __str__(self) -> str:
        return self.name

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class PartialEpisode:
    id: int = None
    air_date: Optional[date] = None
    episode_number: Optional[int] = None
    name: Optional[str] = None
    overview: Optional[str] = None
    production_code: Optional[str] = None
    season_number: Optional[int] = None
    still_path: Optional[str] = None
    vote_average: Optional[float] = None
    vote_count: Optional[int] = None

    def __str__(self) -> str:
        return self.name

    def still_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.still_path}" if self.still_path else None


@dataclass
class KnownFor(PartialMovie, PartialTV):
    ...


@dataclass
class PartialPerson:
    id: int
    profile_path: Optional[str] = None
    adult: Optional[bool] = None
    known_for: Optional[List[KnownFor]] = None
    known_for_department: Optional[str] = None
    gender: Optional[int] = None
    name: Optional[str] = None
    original_name: Optional[str] = None
    popularity: Optional[float] = None
    media_type: Optional[MediaType] = MediaType.person

    def __str__(self) -> str:
        return self.name or self.original_name

    def profile_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.profile_path}" if self.profile_path else None



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialMovie
from themoviedb.schemas._result import ResultWithPage
from themoviedb.schemas.alternative_titles import AlternativeTitles
from themoviedb.schemas.collections import Collection
from themoviedb.schemas.companies import Company
from themoviedb.schemas.countries import Country
from themoviedb.schemas.credits import Credits
from themoviedb.schemas.external_ids import ExternalIDs
from themoviedb.schemas.genres import Genre
from themoviedb.schemas.images import Images
from themoviedb.schemas.keywords import Keywords
from themoviedb.schemas.languages import Language
from themoviedb.schemas.videos import Videos
from themoviedb.schemas.watch_providers import WatchProviders


@dataclass
class Movie(PartialMovie):
    belongs_to_collection: Optional[Collection] = None
    budget: Optional[int] = None
    genres: Optional[List[Genre]] = None
    homepage: Optional[str] = None
    imdb_id: Optional[str] = None
    production_companies: Optional[List[Company]] = None
    production_countries: Optional[List[Country]] = None
    revenue: Optional[int] = None
    runtime: Optional[int] = None
    spoken_languages: Optional[List[Language]] = None
    status: Optional[str] = None
    tagline: Optional[str] = None
    alternative_titles: Optional[AlternativeTitles] = None
    # changes = Optional[] = None  # TODO
    credits: Optional[Credits] = None
    external_ids: Optional[ExternalIDs] = None
    images: Optional[Images] = None
    keywords: Optional[Keywords] = None
    # lists = Optional[] = None  # TODO
    recommendations: Optional[Movies] = None
    # release_dates = Optional[] = None  # TODO
    # reviews = Optional[] = None  # TODO
    similar: Optional[Movies] = None
    # translations = Optional[] = None  # TODO
    videos: Optional[Videos] = None
    watch_providers: Optional[WatchProviders] = None

    @property
    def imdb(self) -> Optional[str]:
        return f"https://www.imdb.com/title/{self.imdb_id}" if self.imdb_id else None


@dataclass
class Movies(ResultWithPage):
    results: Optional[List[PartialMovie]] = None



from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas._partial import PartialTV, PartialSeason, PartialEpisode
from themoviedb.schemas._result import ResultWithPage
from themoviedb.schemas.countries import Country
from themoviedb.schemas.genres import Genre
from themoviedb.schemas.languages import Language
from themoviedb.schemas.networks import Network


@dataclass
class CreatedBy:
    id: Optional[int] = None
    credit_id: Optional[str] = None
    name: Optional[str] = None
    gender: Optional[int] = None
    profile_path: Optional[str] = None

    def __str__(self) -> str:
        return self.name

    def profile_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.profile_path}" if self.profile_path else None


@dataclass
class TV(PartialTV):
    created_by: Optional[List[CreatedBy]] = None
    episode_run_time: Optional[List[int]] = None
    genres: Optional[List[Genre]] = None
    homepage: Optional[str] = None
    in_production: Optional[bool] = None
    languages: Optional[List[str]] = None
    last_air_date: Optional[date] = None
    last_episode_to_air: Optional[PartialEpisode] = None
    next_episode_to_air: Optional[None] = None
    networks: Optional[List[Network]] = None
    number_of_episodes: Optional[int] = None
    number_of_seasons: Optional[int] = None
    production_companies: Optional[List[Network]] = None
    production_countries: Optional[List[Country]] = None
    seasons: Optional[List[PartialSeason]] = None
    spoken_languages: Optional[List[Language]] = None
    status: Optional[str] = None
    tagline: Optional[str] = None
    type: Optional[str] = None


@dataclass
class TVs(ResultWithPage):
    results: Optional[List[PartialTV]] = None



from datetime import date
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialPerson
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Person(PartialPerson):
    birthday: Optional[date] = None
    deathday: Optional[date] = None
    also_known_as: Optional[List[str]] = None
    biography: Optional[str] = None
    place_of_birth: Optional[str] = None
    imdb_id: Optional[str] = None
    homepage: Optional[str] = None

    @property
    def imdb(self) -> Optional[str]:
        return f"https://www.imdb.com/name/{self.imdb_id}" if self.imdb_id else None


@dataclass
class People(ResultWithPage):
    results: Optional[List[Person]] = None
